@@ -3,13 +3,13 @@
 import datetime as dt
 import json
 import logging as lg
-import math
 import re
 import time
 from collections import OrderedDict
 from hashlib import sha1
 from pathlib import Path
 
+import numpy as np
 import requests
 from dateutil import parser as date_parser
 
@@ -33,65 +33,70 @@ def _get_osm_filter(network_type):
     -------
     string
     """
-    # define preset queries to send to the API. specifying way["highway"]
+    # define built-in queries to send to the API. specifying way["highway"]
     # means that all ways returned must have a highway tag. the filters then
     # remove ways by tag/value.
     filters = dict()
 
     # driving: filter out un-drivable roads, service roads, private ways, and
     # anything specifying motor=no. also filter out any non-service roads that
-    # are tagged as providing parking, driveway, private, or emergency-access
-    # services
+    # are tagged as providing certain services
     filters["drive"] = (
-        f'["highway"]["area"!~"yes"]["highway"!~"cycleway|footway|path|pedestrian|steps|track|corridor|'
-        f'elevator|escalator|proposed|construction|bridleway|abandoned|platform|raceway|service"]'
-        f'["motor_vehicle"!~"no"]["motorcar"!~"no"]{settings.default_access}'
-        f'["service"!~"parking|parking_aisle|driveway|private|emergency_access"]'
+        f'["highway"]["area"!~"yes"]{settings.default_access}'
+        f'["highway"!~"abandoned|bridleway|bus_guideway|construction|corridor|cycleway|elevator|'
+        f"escalator|footway|path|pedestrian|planned|platform|proposed|raceway|service|"
+        f'steps|track"]'
+        f'["motor_vehicle"!~"no"]["motorcar"!~"no"]'
+        f'["service"!~"alley|driveway|emergency_access|parking|parking_aisle|private"]'
     )
 
-    # drive+service: allow ways tagged 'service' but filter out certain types of
-    # service ways
+    # drive+service: allow ways tagged 'service' but filter out certain types
     filters["drive_service"] = (
-        f'["highway"]["area"!~"yes"]["highway"!~"cycleway|footway|path|pedestrian|steps|track|'
-        f'corridor|elevator|escalator|proposed|construction|bridleway|abandoned|platform|raceway"]'
-        f'["motor_vehicle"!~"no"]["motorcar"!~"no"]{settings.default_access}'
-        f'["service"!~"parking|parking_aisle|private|emergency_access"]'
+        f'["highway"]["area"!~"yes"]{settings.default_access}'
+        f'["highway"!~"abandoned|bridleway|bus_guideway|construction|corridor|cycleway|elevator|'
+        f'escalator|footway|path|pedestrian|planned|platform|proposed|raceway|steps|track"]'
+        f'["motor_vehicle"!~"no"]["motorcar"!~"no"]'
+        f'["service"!~"emergency_access|parking|parking_aisle|private"]'
     )
 
     # walking: filter out cycle ways, motor ways, private ways, and anything
     # specifying foot=no. allow service roads, permitting things like parking
-    # lot lanes, alleys, etc that you *can* walk on even if they're not exactly
-    # pleasant walks. some cycleways may allow pedestrians, but this filter ignores
-    # such cycleways.
+    # lot lanes, alleys, etc that you *can* walk on even if they're not
+    # exactly pleasant walks. some cycleways may allow pedestrians, but this
+    # filter ignores such cycleways.
     filters["walk"] = (
-        f'["highway"]["area"!~"yes"]["highway"!~"cycleway|motor|proposed|construction|abandoned|'
-        f'platform|raceway"]["foot"!~"no"]["service"!~"private"]{settings.default_access}'
+        f'["highway"]["area"!~"yes"]{settings.default_access}'
+        f'["highway"!~"abandoned|bus_guideway|construction|cycleway|motor|planned|platform|'
+        f'proposed|raceway"]'
+        f'["foot"!~"no"]["service"!~"private"]'
     )
 
     # biking: filter out foot ways, motor ways, private ways, and anything
     # specifying biking=no
     filters["bike"] = (
-        f'["highway"]["area"!~"yes"]["highway"!~"footway|steps|corridor|elevator|'
-        f'escalator|motor|proposed|construction|abandoned|platform|raceway"]'
-        f'["bicycle"!~"no"]["service"!~"private"]{settings.default_access}'
+        f'["highway"]["area"!~"yes"]{settings.default_access}'
+        f'["highway"!~"abandoned|bus_guideway|construction|corridor|elevator|escalator|footway|'
+        f'motor|planned|platform|proposed|raceway|steps"]'
+        f'["bicycle"!~"no"]["service"!~"private"]'
     )
 
     # to download all ways, just filter out everything not currently in use or
     # that is private-access only
     filters["all"] = (
-        f'["highway"]["area"!~"yes"]["highway"!~"proposed|construction|abandoned|platform|raceway"]'
-        f'["service"!~"private"]{settings.default_access}'
+        f'["highway"]["area"!~"yes"]{settings.default_access}'
+        f'["highway"!~"abandoned|construction|planned|platform|proposed|raceway"]'
+        f'["service"!~"private"]'
     )
 
     # to download all ways, including private-access ones, just filter out
     # everything not currently in use
     filters[
         "all_private"
-    ] = '["highway"]["area"!~"yes"]["highway"!~"proposed|construction|abandoned|platform|raceway"]'
+    ] = '["highway"]["area"!~"yes"]["highway"!~"abandoned|construction|planned|platform|proposed|raceway"]'
 
     if network_type in filters:
         osm_filter = filters[network_type]
-    else:
+    else:  # pragma: no cover
         raise ValueError(f'Unrecognized network_type "{network_type}"')
 
     return osm_filter
@@ -283,7 +288,7 @@ def _get_pause(recursive_delay=5, default_duration=60):
         if status_first_token == "Slot":
             utc_time_str = status.split(" ")[3]
             utc_time = date_parser.parse(utc_time_str).replace(tzinfo=None)
-            pause = math.ceil((utc_time - dt.datetime.utcnow()).total_seconds())
+            pause = int(np.ceil((utc_time - dt.datetime.utcnow()).total_seconds()))
             pause = max(pause, 1)
 
         # if first token is 'Currently', it is currently running a query so
@@ -361,7 +366,7 @@ def _create_overpass_query(polygon_coord_str, tags):
 
     # make sure every value in dict is bool, str, or list of str
     error_msg = "tags must be a dict with values of bool, str, or list of str"
-    if not isinstance(tags, dict):
+    if not isinstance(tags, dict):  # pragma: no cover
         raise TypeError(error_msg)
 
     tags_dict = dict()
@@ -374,11 +379,11 @@ def _create_overpass_query(polygon_coord_str, tags):
             tags_dict[key] = [value]
 
         elif isinstance(value, list):
-            if not all(isinstance(s, str) for s in value):
+            if not all(isinstance(s, str) for s in value):  # pragma: no cover
                 raise TypeError(error_msg)
             tags_dict[key] = value
 
-        else:
+        else:  # pragma: no cover
             raise TypeError(error_msg)
 
     # convert the tags dict into a list of {tag:value} dicts
@@ -455,7 +460,7 @@ def _osm_network_download(polygon, network_type, custom_filter):
         f"Got all network data within polygon from API in {len(polygon_coord_strs)} request(s)"
     )
 
-    if settings.cache_only_mode:
+    if settings.cache_only_mode:  # pragma: no cover
         raise CacheOnlyModeInterrupt("settings.cache_only_mode=True")
 
     return response_jsons
@@ -540,7 +545,7 @@ def _osm_place_download(query, by_osmid=False, limit=1, polygon_geojson=1):
             # each time, for caching purposes
             for key in sorted(query):
                 params[key] = query[key]
-        else:
+        else:  # pragma: no cover
             raise TypeError("query must be a dict or a string")
 
     # request the URL, return the JSON
@@ -568,7 +573,7 @@ def nominatim_request(params, request_type="search", pause=1, error_pause=60):
     -------
     response_json : dict
     """
-    if request_type not in {"search", "reverse", "lookup"}:
+    if request_type not in {"search", "reverse", "lookup"}:  # pragma: no cover
         raise ValueError('Nominatim request_type must be "search", "reverse", or "lookup"')
 
     # prepare Nominatim API URL and see if request already exists in cache
@@ -595,9 +600,9 @@ def nominatim_request(params, request_type="search", pause=1, error_pause=60):
         sc = response.status_code
 
         # log the response size and domain
-        size_kb = len(response.content) / 1000.0
+        size_kb = len(response.content) / 1000
         domain = re.findall(r"(?s)//(.*?)/", url)[0]
-        utils.log(f"Downloaded {size_kb:,.1f}KB from {domain}")
+        utils.log(f"Downloaded {size_kb:,.1f}kB from {domain}")
 
         try:
             response_json = response.json()
@@ -663,9 +668,9 @@ def overpass_request(data, pause=None, error_pause=60):
         sc = response.status_code
 
         # log the response size and domain
-        size_kb = len(response.content) / 1000.0
+        size_kb = len(response.content) / 1000
         domain = re.findall(r"(?s)//(.*?)/", url)[0]
-        utils.log(f"Downloaded {size_kb:,.1f}KB from {domain}")
+        utils.log(f"Downloaded {size_kb:,.1f}kB from {domain}")
 
         try:
             response_json = response.json()
