@@ -3,7 +3,9 @@ Download geospatial entities' geometries and attributes from OpenStreetMap.
 
 Retrieve points of interest, building footprints, or any other objects from
 OSM, including their geometries and attribute data, and construct a
-GeoDataFrame of them.
+GeoDataFrame of them. You can use this module to query for nodes, ways, and
+relations (the latter of type "multipolygon" or "boundary" only) by passing a
+dictionary of desired tags/values.
 """
 
 import logging as lg
@@ -22,6 +24,7 @@ from shapely.ops import polygonize
 
 from . import downloader
 from . import geocoder
+from . import osm_xml
 from . import settings
 from . import utils
 from . import utils_geo
@@ -161,7 +164,20 @@ def geometries_from_address(address, tags, dist=1000):
 
 def geometries_from_place(query, tags, which_result=None, buffer_dist=None):
     """
-    Create a GeoDataFrame of OSM entities within the boundaries of a place.
+    Create GeoDataFrame of OSM entities within boundaries of geocodable place(s).
+
+    The query must be geocodable and OSM must have polygon boundaries for the
+    geocode result. If OSM does not have a polygon for this place, you can
+    instead get geometries within it using the geometries_from_address
+    function, which geocodes the place name to a point and gets the geometries
+    within some distance of that point.
+
+    If OSM does have polygon boundaries for this place but you're not finding
+    it, try to vary the query string, pass in a structured query dict, or vary
+    the which_result argument to use a different geocode result. If you know
+    the OSM ID of the place, you can retrieve its boundary polygon using the
+    geocode_to_gdf function, then pass it to the geometries_from_polygon
+    function.
 
     Parameters
     ----------
@@ -181,7 +197,7 @@ def geometries_from_place(query, tags, which_result=None, buffer_dist=None):
         landuse=commercial, and highway=bus_stop.
     which_result : int
         which geocoding result to use. if None, auto-select the first
-        multi/polygon or raise an error if OSM doesn't return one.
+        (Multi)Polygon or raise an error if OSM doesn't return one.
     buffer_dist : float
         distance to buffer around the place geometry, in meters
 
@@ -204,7 +220,7 @@ def geometries_from_place(query, tags, which_result=None, buffer_dist=None):
     elif isinstance(query, list):
         # if it is a list, it contains multiple places to get
         gdf_place = geocoder.geocode_to_gdf(query, buffer_dist=buffer_dist)
-    else:
+    else:  # pragma: no cover
         raise TypeError("query must be dict, string, or list of strings")
 
     # extract the geometry from the GeoDataFrame to use in API query
@@ -259,7 +275,7 @@ def geometries_from_polygon(polygon, tags):
         )
 
     # download the geometry data from OSM
-    response_jsons = downloader._osm_geometry_download(polygon, tags)
+    response_jsons = downloader._osm_geometries_download(polygon, tags)
 
     # create GeoDataFrame from the downloaded data
     gdf = _create_gdf(response_jsons, polygon, tags)
@@ -280,7 +296,7 @@ def geometries_from_xml(filepath, polygon=None, tags=None):
 
     Parameters
     ----------
-    filepath : string
+    filepath : string or pathlib.Path
         path to file containing OSM XML data
     polygon : shapely.geometry.Polygon
         optional geographic boundary to filter objects
@@ -302,7 +318,7 @@ def geometries_from_xml(filepath, polygon=None, tags=None):
     gdf : geopandas.GeoDataFrame
     """
     # transmogrify file of OSM XML data into JSON
-    response_jsons = [downloader._overpass_json_from_file(filepath)]
+    response_jsons = [osm_xml._overpass_json_from_file(filepath)]
 
     # create GeoDataFrame using this response JSON
     gdf = _create_gdf(response_jsons, polygon=polygon, tags=tags)
@@ -333,13 +349,10 @@ def _create_gdf(response_jsons, polygon, tags):
     gdf : geopandas.GeoDataFrame
         GeoDataFrame of geometries and their associated tags
     """
-    # make sure we got data back from the server request(s)
-    element_count = sum([len(rj["elements"]) for rj in response_jsons])
-    utils.log(f"{element_count} elements in the JSON responses (includes every node).")
+    elements_count = sum(len(rj["elements"]) for rj in response_jsons)
 
     # if there are no elements in the responses
-    if element_count < 1:
-
+    if elements_count == 0:
         # create an empty GeoDataFrame
         gdf = gpd.GeoDataFrame()
         gdf["geometry"] = np.nan
@@ -352,8 +365,7 @@ def _create_gdf(response_jsons, polygon, tags):
 
     # else if there were elements in the response
     else:
-        # log creation
-        utils.log("Converting elements to geometries")
+        utils.log(f"Converting {elements_count} elements in JSON responses to geometries")
 
         # Dictionaries to hold nodes and complete geometries
         coords = dict()
@@ -362,6 +374,9 @@ def _create_gdf(response_jsons, polygon, tags):
         # Set to hold the unique IDs of elements that do not have tags
         untagged_element_ids = set()
 
+        # identify which relation types to parse to (multi)polygons
+        relation_types = {"boundary", "multipolygon"}
+
         # extract geometries from the downloaded osm data
         for response_json in response_jsons:
             # Parses the JSON of OSM nodes, ways and (multipolygon) relations
@@ -377,7 +392,7 @@ def _create_gdf(response_jsons, polygon, tags):
                 # with empty tags to the untagged_element_ids set (untagged
                 # nodes are not added to the geometries dict at all)
                 if (element["type"] != "node") and (
-                    ("tags" not in element) or (len(element["tags"]) == 0)
+                    ("tags" not in element) or (not element["tags"])
                 ):
                     untagged_element_ids.add(unique_id)
 
@@ -401,9 +416,9 @@ def _create_gdf(response_jsons, polygon, tags):
 
                 elif (
                     element["type"] == "relation"
-                    and element.get("tags").get("type") == "multipolygon"
+                    and element.get("tags").get("type") in relation_types
                 ):
-                    # Parse all multipolygon relations to multipolygons
+                    # parse relations to (multi)polygons
                     multipolygon = _parse_relation_to_multipolygon(
                         element=element, geometries=geometries
                     )
@@ -536,7 +551,7 @@ def _parse_way_to_linestring_or_polygon(element, coords, polygon_features=_polyg
     if element["nodes"][0] != element["nodes"][-1]:
         try:
             geometry = LineString([(coords[node]["lon"], coords[node]["lat"]) for node in nodes])
-        except KeyError as e:
+        except KeyError as e:  # pragma: no cover
             # XMLs may include geometries that are incomplete, in which case
             # return an empty geometry
             utils.log(
@@ -631,43 +646,40 @@ def _is_closed_way_a_polygon(element, polygon_features=_polygon_features):
             # identify common keys in element's tags and polygon_features dict
             intersecting_keys = element_tags.keys() & polygon_features.keys()
 
-            # if common keys are found
-            if len(intersecting_keys) > 0:
+            # for each key in the intersecting keys (if any found)
+            for key in intersecting_keys:
 
-                # for each key in the intersecting keys
-                for key in intersecting_keys:
+                # Get the key's value from the element's tags
+                key_value = element_tags.get(key)
 
-                    # Get the key's value from the element's tags
-                    key_value = element_tags.get(key)
+                # Determine if the key is for a blocklist or passlist in
+                # polygon_features dict
+                blocklist_or_passlist = polygon_features.get(key).get("polygon")
 
-                    # Determine if the key is for a blocklist or passlist in
-                    # polygon_features dict
-                    blocklist_or_passlist = polygon_features.get(key).get("polygon")
+                # Get values for the key from the polygon_features dict
+                polygon_features_values = polygon_features.get(key).get("values")
 
-                    # Get values for the key from the polygon_features dict
-                    polygon_features_values = polygon_features.get(key).get("values")
+                # if all features with that key should be polygons -> Polygon
+                if blocklist_or_passlist == "all":
+                    is_polygon = True
 
-                    # if all features with that key should be polygons -> Polygon
-                    if blocklist_or_passlist == "all":
+                # if the key is for a blocklist i.e. tags that should not
+                # become Polygons
+                elif blocklist_or_passlist == "blocklist":
+
+                    # if the value for that key in the element is not in
+                    # the blocklist -> Polygon
+                    if key_value not in polygon_features_values:
                         is_polygon = True
 
-                    # if the key is for a blocklist i.e. tags that should not
-                    # become Polygons
-                    elif blocklist_or_passlist == "blocklist":
+                # if the key is for a passlist i.e. specific tags should
+                # become Polygons
+                elif blocklist_or_passlist == "passlist":
 
-                        # if the value for that key in the element is not in
-                        # the blocklist -> Polygon
-                        if key_value not in polygon_features_values:
-                            is_polygon = True
-
-                    # if the key is for a passlist i.e. specific tags should
-                    # become Polygons
-                    elif blocklist_or_passlist == "passlist":
-
-                        # if the value for that key in the element is in the
-                        # passlist -> Polygon
-                        if key_value in polygon_features_values:
-                            is_polygon = True
+                    # if the value for that key in the element is in the
+                    # passlist -> Polygon
+                    if key_value in polygon_features_values:
+                        is_polygon = True
 
     return is_polygon
 
@@ -709,7 +721,7 @@ def _parse_relation_to_multipolygon(element, geometries):
     # may include the relation but not the ways outside the bounding box.
     try:
         member_ways = [geometries[f"way/{member_way_ref}"] for member_way_ref in member_way_refs]
-    except KeyError as e:
+    except KeyError as e:  # pragma: no cover
         utils.log(
             f"{e} was not found in `geometries`.\nThe geometry for "
             f"https://www.openstreetmap.org/{element['type']}/{element['id']} was not created."
@@ -804,7 +816,7 @@ def _assemble_multipolygon_component_polygons(element, geometries):
         for merged_inner_linestring in list(merged_inner_linestrings):
             inner_polygons += polygonize(merged_inner_linestring)
 
-    if len(outer_polygons) == 0:
+    if not outer_polygons:
         utils.log(
             "No outer polygons were created for"
             f" https://www.openstreetmap.org/{element['type']}/{element['id']}"
@@ -844,7 +856,7 @@ def _subtract_inner_polygons_from_outer_polygons(element, outer_polygons, inner_
             if inner_polygon.within(outer_polygon):
                 try:
                     outer_polygon = outer_polygon.difference(inner_polygon)
-                except TopologicalError:
+                except TopologicalError:  # pragma: no cover
                     utils.log(
                         f"relation https://www.openstreetmap.org/relation/{element['id']} caused"
                         " a TopologicalError, trying with zero buffer."
@@ -899,7 +911,7 @@ def _buffer_invalid_geometries(gdf):
         invalid_geometry_filter = ~gdf["geometry"].is_valid
 
         # if there are invalid geometries
-        if sum(invalid_geometry_filter) > 0:
+        if invalid_geometry_filter.any():
 
             # get their unique_ids from the index
             invalid_geometry_ids = gdf.loc[invalid_geometry_filter].index.to_list()
@@ -988,9 +1000,8 @@ def _filter_gdf_by_polygon_and_tags(gdf, polygon, tags):
         # remove columns of all nulls (created by discarded component geometries)
         gdf.dropna(axis="columns", how="all", inplace=True)
 
-        # reset the index keeping the unique ids
-        gdf.reset_index(inplace=True)
-        # rename the new 'index' column to 'unique_id'
-        gdf.rename(columns={"index": "unique_id"}, inplace=True)
-
+    # multi-index gdf by element_type and osmid then return
+    idx_cols = ["element_type", "osmid"]
+    if all(c in gdf.columns for c in idx_cols):
+        gdf = gdf.set_index(idx_cols)
     return gdf
