@@ -1,6 +1,6 @@
 """Graph creation functions."""
 
-from itertools import groupby
+import itertools
 
 import networkx as nx
 from shapely.geometry import MultiPolygon
@@ -9,6 +9,7 @@ from shapely.geometry import Polygon
 from . import distance
 from . import downloader
 from . import geocoder
+from . import osm_xml
 from . import projection
 from . import settings
 from . import simplification
@@ -45,11 +46,10 @@ def graph_from_bbox(
         eastern longitude of bounding box
     west : float
         western longitude of bounding box
-    network_type : string
-        what type of street network to get if custom_filter is None. One of
-        'walk', 'bike', 'drive', 'drive_service', 'all', or 'all_private'.
+    network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
+        what type of street network to get if custom_filter is None
     simplify : bool
-        if True, simplify the graph topology with the simplify_graph function
+        if True, simplify graph topology with the `simplify_graph` function
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -60,7 +60,7 @@ def graph_from_bbox(
         if True, buffer 500m to get a graph larger than requested, then
         simplify, then truncate it to requested spatial boundaries
     custom_filter : string
-        a custom network filter to be used instead of the network_type presets,
+        a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
         in a network_type that is in settings.bidirectional_network_types if
         you want graph to be fully bi-directional.
@@ -113,15 +113,15 @@ def graph_from_point(
     dist : int
         retain only those nodes within this many meters of the center of the
         graph, with distance determined according to dist_type argument
-    dist_type : string
-        {'network', 'bbox'} if 'bbox', retain only those nodes within a bounding
-        box of the distance parameter. if 'network', retain only those nodes
-        within some network distance from the center-most node.
-    network_type : string
-        what type of street network to get if custom_filter is None. One of
-        'walk', 'bike', 'drive', 'drive_service', 'all', or 'all_private'.
+    dist_type : string {"network", "bbox"}
+        if "bbox", retain only those nodes within a bounding box of the
+        distance parameter. if "network", retain only those nodes within some
+        network distance from the center-most node (requires that scikit-learn
+        is installed as an optional dependency).
+    network_type : string, {"all_private", "all", "bike", "drive", "drive_service", "walk"}
+        what type of street network to get if custom_filter is None
     simplify : bool
-        if True, simplify the graph topology with the simplify_graph function
+        if True, simplify graph topology with the `simplify_graph` function
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -132,7 +132,7 @@ def graph_from_point(
         if True, buffer 500m to get a graph larger than requested, then
         simplify, then truncate it to requested spatial boundaries
     custom_filter : string
-        a custom network filter to be used instead of the network_type presets,
+        a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
         in a network_type that is in settings.bidirectional_network_types if
         you want graph to be fully bi-directional.
@@ -146,11 +146,10 @@ def graph_from_point(
     You can configure the Overpass server timeout, memory allocation, and
     other custom settings via ox.config().
     """
-    if dist_type not in {"bbox", "network"}:
+    if dist_type not in {"bbox", "network"}:  # pragma: no cover
         raise ValueError('dist_type must be "bbox" or "network"')
 
-    # create a bounding box from the center point and the distance in each
-    # direction
+    # create bounding box from center point and distance in each direction
     north, south, east, west = utils_geo.bbox_from_point(center_point, dist)
 
     # create a graph from the bounding box
@@ -167,11 +166,11 @@ def graph_from_point(
         custom_filter=custom_filter,
     )
 
-    # if network dist_type is network, find the node in the graph nearest to
-    # the center point, and truncate the graph by network distance from node
     if dist_type == "network":
-        centermost_node = distance.get_nearest_node(G, center_point)
-        G = truncate.truncate_graph_dist(G, centermost_node, max_dist=dist)
+        # if dist_type is network, find node in graph nearest to center point
+        # then truncate graph by network dist from it
+        node = distance.nearest_nodes(G, X=[center_point[1]], Y=[center_point[0]])[0]
+        G = truncate.truncate_graph_dist(G, node, max_dist=dist)
 
     utils.log(f"graph_from_point returned graph with {len(G)} nodes and {len(G.edges)} edges")
     return G
@@ -200,16 +199,15 @@ def graph_from_address(
     dist : int
         retain only those nodes within this many meters of the center of the
         graph
-    dist_type : string
-        {'network', 'bbox'} if 'bbox', retain only those nodes within a bounding
-        box of the distance parameter.
-        if 'network', retain only those nodes within some network distance from
-        the center-most node.
-    network_type : string
-        what type of street network to get if custom_filter is None. One of
-        'walk', 'bike', 'drive', 'drive_service', 'all', or 'all_private'.
+    dist_type : string {"network", "bbox"}
+        if "bbox", retain only those nodes within a bounding box of the
+        distance parameter. if "network", retain only those nodes within some
+        network distance from the center-most node (requires that scikit-learn
+        is installed as an optional dependency).
+    network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
+        what type of street network to get if custom_filter is None
     simplify : bool
-        if True, simplify the graph topology with the simplify_graph function
+        if True, simplify graph topology with the `simplify_graph` function
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -222,7 +220,7 @@ def graph_from_address(
         if True, buffer 500m to get a graph larger than requested, then
         simplify, then truncate it to requested spatial boundaries
     custom_filter : string
-        a custom network filter to be used instead of the network_type presets,
+        a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
         in a network_type that is in settings.bidirectional_network_types if
         you want graph to be fully bi-directional.
@@ -275,24 +273,24 @@ def graph_from_place(
 
     The query must be geocodable and OSM must have polygon boundaries for the
     geocode result. If OSM does not have a polygon for this place, you can
-    instead get its street network using the graph_from_address function, which
-    geocodes the place name to a point and gets the network within some distance
-    of that point. Alternatively, you might try to vary the which_result
-    parameter to use a different geocode result. For example, the first geocode
-    result (ie, the default) might resolve to a point geometry, but the second
-    geocode result for this query might resolve to a polygon, in which case you
-    can use graph_from_place with which_result=2. which_result=None will
-    auto-select the first multi/polygon among the geocoding results.
+    instead get its street network using the graph_from_address function,
+    which geocodes the place name to a point and gets the network within some
+    distance of that point.
+
+    If OSM does have polygon boundaries for this place but you're not finding
+    it, try to vary the query string, pass in a structured query dict, or vary
+    the which_result argument to use a different geocode result. If you know
+    the OSM ID of the place, you can retrieve its boundary polygon using the
+    geocode_to_gdf function, then pass it to the graph_from_polygon function.
 
     Parameters
     ----------
     query : string or dict or list
         the query or queries to geocode to get place boundary polygon(s)
-    network_type : string
-        what type of street network to get if custom_filter is None. One of
-        'walk', 'bike', 'drive', 'drive_service', 'all', or 'all_private'.
+    network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
+        what type of street network to get if custom_filter is None
     simplify : bool
-        if True, simplify the graph topology with the simplify_graph function
+        if True, simplify graph topology with the `simplify_graph` function
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -301,14 +299,14 @@ def graph_from_place(
         node's neighbors is within the polygon
     which_result : int
         which geocoding result to use. if None, auto-select the first
-        multi/polygon or raise an error if OSM doesn't return one.
+        (Multi)Polygon or raise an error if OSM doesn't return one.
     buffer_dist : float
         distance to buffer around the place geometry, in meters
     clean_periphery : bool
         if True, buffer 500m to get a graph larger than requested, then
         simplify, then truncate it to requested spatial boundaries
     custom_filter : string
-        a custom network filter to be used instead of the network_type presets,
+        a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
         in a network_type that is in settings.bidirectional_network_types if
         you want graph to be fully bi-directional.
@@ -324,15 +322,15 @@ def graph_from_place(
     """
     # create a GeoDataFrame with the spatial boundaries of the place(s)
     if isinstance(query, (str, dict)):
-        # if it is a string (place name) or dict (structured place query), then
-        # it is a single place
+        # if it is a string (place name) or dict (structured place query),
+        # then it is a single place
         gdf_place = geocoder.geocode_to_gdf(
             query, which_result=which_result, buffer_dist=buffer_dist
         )
     elif isinstance(query, list):
         # if it is a list, it contains multiple places to get
         gdf_place = geocoder.geocode_to_gdf(query, buffer_dist=buffer_dist)
-    else:
+    else:  # pragma: no cover
         raise TypeError("query must be dict, string, or list of strings")
 
     # extract the geometry from the GeoDataFrame to use in API query
@@ -369,13 +367,12 @@ def graph_from_polygon(
     Parameters
     ----------
     polygon : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
-        the shape to get network data within. coordinates should be in units of
-        latitude-longitude degrees.
-    network_type : string
-        what type of street network to get if custom_filter is None. One of
-        'walk', 'bike', 'drive', 'drive_service', 'all', or 'all_private'.
+        the shape to get network data within. coordinates should be in
+        unprojected latitude-longitude degrees (EPSG:4326).
+    network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
+        what type of street network to get if custom_filter is None
     simplify : bool
-        if True, simplify the graph topology with the simplify_graph function
+        if True, simplify graph topology with the `simplify_graph` function
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -386,7 +383,7 @@ def graph_from_polygon(
         if True, buffer 500m to get a graph larger than requested, then
         simplify, then truncate it to requested spatial boundaries
     custom_filter : string
-        a custom network filter to be used instead of the network_type presets,
+        a custom ways filter to be used instead of the network_type presets
         e.g., '["power"~"line"]' or '["highway"~"motorway|trunk"]'. Also pass
         in a network_type that is in settings.bidirectional_network_types if
         you want graph to be fully bi-directional.
@@ -402,9 +399,9 @@ def graph_from_polygon(
     """
     # verify that the geometry is valid and is a shapely Polygon/MultiPolygon
     # before proceeding
-    if not polygon.is_valid:
+    if not polygon.is_valid:  # pragma: no cover
         raise ValueError("The geometry to query within is invalid")
-    if not isinstance(polygon, (Polygon, MultiPolygon)):
+    if not isinstance(polygon, (Polygon, MultiPolygon)):  # pragma: no cover
         raise TypeError(
             "Geometry must be a shapely Polygon or MultiPolygon. If you requested "
             "graph from place name, make sure your query resolves to a Polygon or "
@@ -420,10 +417,10 @@ def graph_from_polygon(
         poly_buff, _ = projection.project_geometry(poly_proj_buff, crs=crs_utm, to_latlong=True)
 
         # download the network data from OSM within buffered polygon
-        response_jsons = downloader._osm_net_download(poly_buff, network_type, custom_filter)
-        bidirectional = network_type in settings.bidirectional_network_types
+        response_jsons = downloader._osm_network_download(poly_buff, network_type, custom_filter)
 
         # create buffered graph from the downloaded data
+        bidirectional = network_type in settings.bidirectional_network_types
         G_buff = _create_graph(response_jsons, retain_all=True, bidirectional=bidirectional)
 
         # truncate buffered graph to the buffered polygon and retain_all for
@@ -443,18 +440,19 @@ def graph_from_polygon(
         # intersection just outside the polygon
         G = truncate.truncate_graph_polygon(G_buff, polygon, retain_all, truncate_by_edge)
 
-        # count how many street segments in buffered graph emanate from each
+        # count how many physical streets in buffered graph connect to each
         # intersection in un-buffered graph, to retain true counts for each
         # intersection, even if some of its neighbors are outside the polygon
-        G.graph["streets_per_node"] = utils_graph.count_streets_per_node(G_buff, nodes=G.nodes())
+        spn = utils_graph.count_streets_per_node(G_buff, nodes=G.nodes)
+        nx.set_node_attributes(G, values=spn, name="street_count")
 
     # if clean_periphery=False, just use the polygon as provided
     else:
         # download the network data from OSM
-        response_jsons = downloader._osm_net_download(polygon, network_type, custom_filter)
-        bidirectional = network_type in settings.bidirectional_network_types
+        response_jsons = downloader._osm_network_download(polygon, network_type, custom_filter)
 
         # create graph from the downloaded data
+        bidirectional = network_type in settings.bidirectional_network_types
         G = _create_graph(response_jsons, retain_all=True, bidirectional=bidirectional)
 
         # truncate the graph to the extent of the polygon
@@ -472,16 +470,16 @@ def graph_from_polygon(
 
 def graph_from_xml(filepath, bidirectional=False, simplify=True, retain_all=False):
     """
-    Create a graph from data in an OSM-formatted XML file.
+    Create a graph from data in a .osm formatted XML file.
 
     Parameters
     ----------
-    filepath : string
+    filepath : string or pathlib.Path
         path to file containing OSM XML data
     bidirectional : bool
         if True, create bi-directional edges for one-way streets
     simplify : bool
-        if True, simplify the graph topology with the simplify_graph function
+        if True, simplify graph topology with the `simplify_graph` function
     retain_all : bool
         if True, return the entire graph even if it is not connected.
         otherwise, retain only the largest weakly connected component.
@@ -491,7 +489,7 @@ def graph_from_xml(filepath, bidirectional=False, simplify=True, retain_all=Fals
     G : networkx.MultiDiGraph
     """
     # transmogrify file of OSM XML data into JSON
-    response_jsons = [downloader._overpass_json_from_file(filepath)]
+    response_jsons = [osm_xml._overpass_json_from_file(filepath)]
 
     # create graph using this response JSON
     G = _create_graph(response_jsons, bidirectional=bidirectional, retain_all=retain_all)
@@ -508,8 +506,9 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
     """
     Create a networkx MultiDiGraph from Overpass API responses.
 
-    Add length in meters (great-circle distance between endpoints) to all
-    of the graph's (unsimplified, straight-line) edges via add_edge_lengths.
+    Adds length attributes in meters (great-circle distance between endpoints)
+    to all of the graph's (pre-simplified, straight-line) edges via the
+    `distance.add_edge_lengths` function.
 
     Parameters
     ----------
@@ -528,8 +527,7 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
     utils.log("Creating graph from downloaded OSM data...")
 
     # make sure we got data back from the server request(s)
-    element_counts = [len(rj["elements"]) for rj in response_jsons]
-    if sum(element_counts) < 1:
+    if not any(rj["elements"] for rj in response_jsons):  # pragma: no cover
         raise EmptyOverpassResponse("There are no data elements in the response JSON")
 
     # create the graph as a MultiDiGraph and set its meta-attributes
@@ -561,9 +559,9 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
 
     utils.log(f"Created graph with {len(G)} nodes and {len(G.edges)} edges")
 
-    # add length (great circle distance between nodes) attribute to each edge
+    # add length (great-circle distance between nodes) attribute to each edge
     if len(G.edges) > 0:
-        G = utils_graph.add_edge_lengths(G)
+        G = distance.add_edge_lengths(G)
 
     return G
 
@@ -581,7 +579,7 @@ def _convert_node(element):
     -------
     node : dict
     """
-    node = {"y": element["lat"], "x": element["lon"], "osmid": element["id"]}
+    node = {"y": element["lat"], "x": element["lon"]}
     if "tags" in element:
         for useful_tag in settings.useful_tags_node:
             if useful_tag in element["tags"]:
@@ -605,8 +603,7 @@ def _convert_path(element):
     path = {"osmid": element["id"]}
 
     # remove any consecutive duplicate elements in the list of nodes
-    grouped_list = groupby(element["nodes"])
-    path["nodes"] = [group[0] for group in grouped_list]
+    path["nodes"] = [group[0] for group in itertools.groupby(element["nodes"])]
 
     if "tags" in element:
         for useful_tag in settings.useful_tags_way:
