@@ -1,7 +1,8 @@
 """Geospatial utility functions."""
 
-import math
+import warnings
 
+import networkx as nx
 import numpy as np
 from shapely.geometry import LineString
 from shapely.geometry import MultiLineString
@@ -14,40 +15,99 @@ from shapely.ops import split
 from . import projection
 from . import settings
 from . import utils
+from . import utils_graph
+
+
+def sample_points(G, n):
+    """
+    Randomly sample points constrained to a spatial graph.
+
+    This generates a graph-constrained uniform random sample of points. Unlike
+    typical spatially uniform random sampling, this method accounts for the
+    graph's geometry. And unlike equal-length edge segmenting, this method
+    guarantees uniform randomness.
+
+    Parameters
+    ----------
+    G : networkx.MultiGraph
+        graph to sample points from; should be undirected (to not oversample
+        bidirectional edges) and projected (for accurate point interpolation)
+    n : int
+        how many points to sample
+
+    Returns
+    -------
+    points : geopandas.GeoSeries
+        the sampled points, multi-indexed by (u, v, key) of the edge from
+        which each point was drawn
+    """
+    if nx.is_directed(G):  # pragma: no cover
+        warnings.warn("graph should be undirected to not oversample bidirectional edges")
+    gdf_edges = utils_graph.graph_to_gdfs(G, nodes=False)[["geometry", "length"]]
+    weights = gdf_edges["length"] / gdf_edges["length"].sum()
+    idx = np.random.choice(gdf_edges.index, size=n, p=weights)
+    lines = gdf_edges.loc[idx, "geometry"]
+    return lines.interpolate(np.random.rand(n), normalized=True)
+
+
+def interpolate_points(geom, dist):
+    """
+    Interpolate evenly spaced points along a LineString.
+
+    The spacing is approximate because the LineString's length may not be
+    evenly divisible by it.
+
+    Parameters
+    ----------
+    geom : shapely.geometry.LineString
+        a LineString geometry
+    dist : float
+        spacing distance between interpolated points, in same units as `geom`.
+        smaller values generate more points.
+
+    Yields
+    ------
+    points : generator
+        a generator of (x, y) tuples of the interpolated points' coordinates
+    """
+    if isinstance(geom, LineString):
+        num_vert = max(round(geom.length / dist), 1)
+        for n in range(num_vert + 1):
+            point = geom.interpolate(n / num_vert, normalized=True)
+            yield point.x, point.y
+    else:  # pragma: no cover
+        raise TypeError(f"unhandled geometry type {geom.geom_type}")
 
 
 def redistribute_vertices(geom, dist):
     """
-    Redistribute the vertices on a projected LineString or MultiLineString.
-
-    The distance argument is only approximate since the total distance of the
-    linestring may not be a multiple of the preferred distance. This function
-    works on only (Multi)LineString geometry types.
+    Do not use, deprecated.
 
     Parameters
     ----------
     geom : shapely.geometry.LineString or shapely.geometry.MultiLineString
-        a Shapely geometry (should be projected)
+        deprecated, do not use
     dist : float
-        spacing length along edges. Units are same as the geom: degrees for
-        unprojected geometries and meters for projected geometries. The
-        smaller the dist value, the more points are created.
+        deprecated, do not use
 
     Returns
     -------
     list or shapely.geometry.MultiLineString
-        the redistributed vertices as a list if geom is a LineString or
-        MultiLineString if geom is a MultiLineString
     """
+    msg = (
+        "The `redistribute_vertices` function has been deprecated and will be removed in a "
+        "future release. Use the more efficient `utils_geo.interpolate_points` instead."
+    )
+    warnings.warn(msg)
     if geom.geom_type == "LineString":
-        num_vert = int(round(geom.length / dist))
+        num_vert = round(geom.length / dist)
         if num_vert == 0:
             num_vert = 1
-        return [geom.interpolate(float(n) / num_vert, normalized=True) for n in range(num_vert + 1)]
+        return [geom.interpolate(n / num_vert, normalized=True) for n in range(num_vert + 1)]
     elif geom.geom_type == "MultiLineString":
         parts = [redistribute_vertices(part, dist) for part in geom]
         return type(geom)([p for p in parts if not p])
-    else:
+    else:  # pragma: no cover
         raise ValueError(f"unhandled geometry {geom.geom_type}")
 
 
@@ -64,21 +124,17 @@ def _round_polygon_coords(p, precision):
 
     Returns
     -------
-    new_poly : shapely.geometry.Polygon
-        the polygon with rounded coordinates
+    shapely.geometry.Polygon
     """
-    # round the coordinates of the Polygon exterior
-    new_exterior = [[round(x, precision) for x in c] for c in p.exterior.coords]
+    # round coords of Polygon exterior
+    shell = [[round(x, precision) for x in c] for c in p.exterior.coords]
 
-    # round the coordinates of the (possibly multiple, possibly none) Polygon interior(s)
-    new_interiors = []
-    for interior in p.interiors:
-        new_interiors.append([[round(x, precision) for x in c] for c in interior.coords])
+    # round coords of (possibly multiple, possibly none) Polygon interior(s)
+    holes = [[[round(x, precision) for x in c] for c in i.coords] for i in p.interiors]
 
-    # construct a new Polygon with the rounded coordinates
-    # buffer by zero to clean self-touching or self-crossing polygons
-    new_poly = Polygon(shell=new_exterior, holes=new_interiors).buffer(0)
-    return new_poly
+    # construct new Polygon with rounded coordinates and buffer by zero to
+    # clean self-touching or self-crossing polygons
+    return Polygon(shell=shell, holes=holes).buffer(0)
 
 
 def _round_multipolygon_coords(mp, precision):
@@ -204,7 +260,7 @@ def round_geometry_coords(geom, precision):
     elif isinstance(geom, MultiPolygon):
         return _round_multipolygon_coords(geom, precision)
 
-    else:
+    else:  # pragma: no cover
         raise TypeError(f"cannot round coordinates of unhandled geometry type: {type(geom)}")
 
 
@@ -227,20 +283,20 @@ def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
 
     Returns
     -------
-    geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
+    geometry : shapely.geometry.MultiPolygon
     """
     if max_query_area_size is None:
         max_query_area_size = settings.max_query_area_size
 
     # let the linear length of the quadrats (with which to subdivide the
     # geometry) be the square root of max area size
-    quadrat_width = math.sqrt(max_query_area_size)
+    quadrat_width = np.sqrt(max_query_area_size)
 
-    if not isinstance(geometry, (Polygon, MultiPolygon)):
+    if not isinstance(geometry, (Polygon, MultiPolygon)):  # pragma: no cover
         raise TypeError("Geometry must be a shapely Polygon or MultiPolygon")
 
-    # if geometry is a MultiPolygon OR a single Polygon whose area exceeds the
-    # max size, get the convex hull around the geometry
+    # if geometry is either 1) a Polygon whose area exceeds the max size, or
+    # 2) a MultiPolygon, then get the convex hull around the geometry
     if isinstance(geometry, MultiPolygon) or (
         isinstance(geometry, Polygon) and geometry.area > max_query_area_size
     ):
@@ -271,26 +327,22 @@ def _get_polygons_coordinates(geometry):
     -------
     polygon_coord_strs : list
     """
-    # extract the exterior coordinates of the geometry to pass to the API later
-    polygons_coords = []
-    if isinstance(geometry, Polygon):
-        x, y = geometry.exterior.xy
-        polygons_coords.append(list(zip(x, y)))
-    elif isinstance(geometry, MultiPolygon):
-        for polygon in geometry:
-            x, y = polygon.exterior.xy
-            polygons_coords.append(list(zip(x, y)))
-    else:
-        raise TypeError("Geometry must be a shapely Polygon or MultiPolygon")
+    if not isinstance(geometry, MultiPolygon):  # pragma: no cover
+        raise TypeError("Geometry must be a shapely MultiPolygon")
 
-    # convert the exterior coordinates of the polygon(s) to the string format
-    # the API expects
+    # extract geometry's exterior coords
+    polygons_coords = []
+    for polygon in geometry:
+        x, y = polygon.exterior.xy
+        polygons_coords.append(list(zip(x, y)))
+
+    # convert exterior coords to the string format the API expects
     polygon_coord_strs = []
     for coords in polygons_coords:
         s = ""
         separator = " "
         for coord in list(coords):
-            # round floating point lats and longs to 6 decimal places (ie, ~100 mm),
+            # round floating point lats and longs to 6 decimals (ie, ~100 mm)
             # so we can hash and cache strings consistently
             s = f"{s}{separator}{coord[1]:.6f}{separator}{coord[0]:.6f}"
         polygon_coord_strs.append(s.strip(separator))
@@ -319,8 +371,8 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
     """
     # create n evenly spaced points between the min and max x and y bounds
     west, south, east, north = geometry.bounds
-    x_num = math.ceil((east - west) / quadrat_width) + 1
-    y_num = math.ceil((north - south) / quadrat_width) + 1
+    x_num = int(np.ceil((east - west) / quadrat_width) + 1)
+    y_num = int(np.ceil((north - south) / quadrat_width) + 1)
     x_points = np.linspace(west, east, num=max(x_num, min_num))
     y_points = np.linspace(south, north, num=max(y_num, min_num))
 
@@ -408,11 +460,11 @@ def bbox_from_point(point, dist=1000, project_utm=False, return_crs=False):
     tuple
         (north, south, east, west) or (north, south, east, west, crs_proj)
     """
-    earth_radius = 6371009  # meters
+    earth_radius = 6_371_009  # meters
     lat, lng = point
 
-    delta_lat = (dist / earth_radius) * (180 / math.pi)
-    delta_lng = (dist / earth_radius) * (180 / math.pi) / math.cos(lat * math.pi / 180)
+    delta_lat = (dist / earth_radius) * (180 / np.pi)
+    delta_lng = (dist / earth_radius) * (180 / np.pi) / np.cos(lat * np.pi / 180)
     north = lat + delta_lat
     south = lat - delta_lat
     east = lng + delta_lng
